// =================================================================================
//
//	ActiveX.h				1996-97 Microsoft Corporation All rights reserved.
//
//	ActiveX type definitions
//
// =================================================================================

#ifndef _H_ActiveX
#define _H_ActiveX
#pragma once


typedef enum
{
	EmptyFocusSet =	0x00000000,
	KeyboardFocus =	0x00000001,
	ModalFocus =	0x00000002,
	FullFocusSet =	0x00000003,
	PadFocusSet = 	0xffffffff			//	insure enum takes 32 bits
}
FocusSet;

typedef enum
{
	BeginFocusCommand = 0,				//	enum boundary value
	RequestReleaseCommand,
	ReleaseCommand,
	TakeNextCommand,
	TakePrevCommand,
	EndFocusCommand,					//	enum boundary value
	PadFocusCommand = 0xffffffff		//	insure enum takes 32 bits
}
FocusCommand;

typedef enum
{
	BeginFindBySpec = 0,				//	enum boundary value
	FindByAny,
	FindByName,
	FindByClassID,
	EndFindBySpec,						//	enum boundary value
	PadFindBySpec = 0xffffffff			//	insure enum takes 32 bits
}
FindBySpec;

typedef struct
{
	FindBySpec	FindBy;
	union u
	{
		Char8	Name[255];
		CLSID	ClassID;
	};
}
SearchSpec;

typedef enum
{
	DepthImmediate =	0x0000,
	DepthParents =		0x0001,
	DepthChildren =		0x0002,
	DepthSiblings =		0x0004
}
DepthSpec;

typedef enum
{
	BeginContextCommand = 0,			//	enum boundary value
	AddContext,
	RemoveContext,
	UpdateContext,
	ActivateContext,
	DeactivateContext,
	EndContextCommand,					//	enum boundary value
	PadContextCommand = 0xffffffff		//	insure enum takes 32 bits
}
ContextCommand;

typedef enum
{
	InvalidContextID = 0xffffffff
}
SpecialContextIDs;

typedef enum
{
	RemoveAllIdlers = -3,
	RemoveIdler = -2,
	IdleAfterAllEvents = -1,
	IdleAfterNullEvents = 0
}
SpecialIdleTickCounts;

typedef enum
{
	BeginHLinkNavigate = 0,				//	enum boundary value
	NavigateURL =  1,					//	Navigate to specified URL
	NavigateBack = 2,					//	Navigate backward in navigation stack  
	NavigateForward = 3,				//	Navigate forward in navigation stack
	EndHLinkNavigate,					//	enum boundary value
	PadHLinkNavigate = 0xffffffff		//	insure enum takes 32 bits
}
HLinkNavigate;

typedef enum
{
	BeginHLinkTarget = 0,				//	enum boundary value
	TargetCurrent = 1,					//	Display in current frame/window
	TargetNewWindow = 2,				//	Display in new window
	EndHLinkTarget,						//	enum boundary value
	PadHLinkTarget = 0xffffffff			//	insure enum takes 32 bits
}
HLinkTarget;

typedef enum
{
	BeginChangeType = 0,
	ViewChange,						// General View change
	UsedAreaChange,					// Used Area changed
	DataChange,							// Data changed, I'm dirty
	PadChangeType = 0xffffffff			// insure enum takes 32 bits
}
ChangeType;

typedef void ChangeData;				// Currently undefined

typedef enum
{
	BeginMouseEventType = 0,
	MouseDown,
	MouseUp,
	MouseDblClick,
	MouseStillDown,
	MouseEnter,
	MouseLeave,
	EndMouseEventType,
	PadMouseEventType = 0xffffffff		//	insure enum takes 32 bits
}
MouseEventType;

typedef enum
{
	BeginKeyEventType = 0,
	KeyDown,
	KeyUp,
	AutoKey,
	EndKeyEventType,
	PadKeyEventType = 0xffffffff		//	insure enum takes 32 bits
}
KeyEventType;

typedef enum
{
	BeginActivateEventType = 0,
	AppActivate,
	AppDeactivate,
	WindowActivate,
	WindowDeactivate,
	EndActivateEventType,
	PadActivateEventType = 0xffffffff	//	insure enum takes 32 bits
}
ActivateEventType;



/////////////////////////////////////////////////////////////////////////////
// IContainer interface

DECLARE_INTERFACE_(IContainer, IUnknown)
{
	BEGIN_INTERFACE

	// *** IUnknown methods ***
	STDMETHOD (QueryInterface) (THIS_ REFIID inRefIID, void **outObj) PURE;
	STDMETHOD_(Uint32,AddRef) (THIS)  PURE;
	STDMETHOD_(Uint32,Release) (THIS) PURE;

	// **** IContainer methods ***
	STDMETHOD (EnumControls) (THIS_ SearchSpec* inSpec, Uint32 inSearchDepth, IEnumUnknown** outEnumerator) PURE;
};


/////////////////////////////////////////////////////////////////////////////
// IHLinkBasic interface

DECLARE_INTERFACE_(IHLinkBasic, IUnknown)
{
	BEGIN_INTERFACE

	// *** IUnknown methods ***
	STDMETHOD (QueryInterface) (THIS_ REFIID inRefIID, void **outObj) PURE;
	STDMETHOD_(Uint32,AddRef) (THIS)  PURE;
	STDMETHOD_(Uint32,Release) (THIS) PURE;

	// **** IHLinkBasic methods ***
	STDMETHOD(GotoURL)(THIS_ HLinkTarget inTarget, Char8* inURL) PURE;
	STDMETHOD(GoBack)(THIS) PURE;
	STDMETHOD(GoForward)(THIS) PURE;
	STDMETHOD(CanGoBack)(THIS) PURE;
	STDMETHOD(CanGoForward)(THIS) PURE;
};



/////////////////////////////////////////////////////////////////////////////
// IThreadScheduler interface
DECLARE_INTERFACE_(IThreadScheduler, IUnknown)
{
	BEGIN_INTERFACE

	// *** IUnknown methods ***
	STDMETHOD (QueryInterface) (THIS_ REFIID inRefIID, void **outObj) PURE;
	STDMETHOD_(Uint32,AddRef) (THIS)  PURE;
	STDMETHOD_(Uint32,Release) (THIS) PURE;

	// **** IThreadScheduler methods ***
	STDMETHOD(GetScheduler)(THIS_ ProcPtr* outScheduler) PURE;
};


/////////////////////////////////////////////////////////////////////////////
// IContainerSite interface

DECLARE_INTERFACE_(IContainerSite, IUnknown)
{ 
	BEGIN_INTERFACE

	// *** IUnknown methods ***
	STDMETHOD (QueryInterface)(THIS_ REFIID inRefID, void** outObj) PURE;
	STDMETHOD_(ULONG,AddRef)(THIS)  PURE;
	STDMETHOD_(ULONG,Release)(THIS) PURE;

	// **** IContainerSite methods ***
	STDMETHOD (GetContainer)(THIS_ IContainer** outContainer) PURE;
	STDMETHOD (RequestFocus) (THIS_ Boolean8 inAcquire, FocusSet inFocus) PURE;
	STDMETHOD (RequestSizeChange) (THIS_ PlatformPoint* ioSize) PURE;
	STDMETHOD (OnChange) (THIS_ ChangeType inChangeType) PURE;
	STDMETHOD (AcquireContext) (THIS_ Uint32 inContextID, DrawContext* outContext) PURE;
	STDMETHOD (ReleaseContext) (THIS_ DrawContext* inContext) PURE;
	STDMETHOD (SetIdleTime) (THIS_ Int32 inWaitTicks, Uint32 inRefCon) PURE;
};


/////////////////////////////////////////////////////////////////////////////
// IControl interface

DECLARE_INTERFACE_(IControl, IUnknown)
{ 
	BEGIN_INTERFACE

	// *** IUnknown methods ***
	STDMETHOD (QueryInterface)(THIS_ REFIID inRefID, void** outObj) PURE;
	STDMETHOD_(ULONG,AddRef)(THIS)  PURE;
	STDMETHOD_(ULONG,Release)(THIS) PURE;

	// *** IControl methods ***
	STDMETHOD (Draw) (THIS_ DrawContext* inContext) PURE;
	STDMETHOD (OnContextChange)(THIS_ UInt32 ContextID, ContextCommand inCommand) PURE;
	STDMETHOD (GetID)(THIS_ Int32 inBufferSize, Char8* outID) PURE;
	STDMETHOD (GetUsedArea)(THIS_ PlatformRegion* outUsedArea)  PURE;
	STDMETHOD (SetFocus)(THIS_ FocusCommand inCommand, FocusSet inFocus)  PURE;
	STDMETHOD (DoMouse)(THIS_ MouseEventType inMouseET, PlatformEvent* inEvent)  PURE;
	STDMETHOD (DoKey)(THIS_ KeyEventType inKeyET, Char8 inChar, PlatformEvent* inEvent) PURE;
	STDMETHOD (DoActivate)(THIS_ ActivateEventType inActiveET, UInt32 ContextID, PlatformEvent* inEvent)  PURE;
	STDMETHOD (DoSystemEvent)(THIS_ PlatformEvent* inEvent)  PURE;
	STDMETHOD (DoIdle)(THIS_ Uint32 IdleRefCon)  PURE;
};


/////////////////////////////////////////////////////////////////////////////
// IStandardEvents interface
// Outgoing interface for the standard control events

DECLARE_INTERFACE_(IStandardEvents, IUnknown)
{
	BEGIN_INTERFACE

	// *** IUnknown methods ***
	STDMETHOD (QueryInterface)(THIS_ REFIID inRefID, void** outObj) PURE;
	STDMETHOD_(ULONG,AddRef)(THIS)  PURE;
	STDMETHOD_(ULONG,Release)(THIS) PURE;

	// *** IStandardEvents methods ***
	STDMETHOD (OnDoubleClick)(THIS_ IUnknown* inSource, PlatformEvent* inEvent) PURE;
	STDMETHOD (OnKeyDown)(THIS_ IUnknown* inSource, PlatformEvent* inEvent) PURE;
	STDMETHOD (OnAutoKey)(THIS_ IUnknown* inSource, PlatformEvent* inEvent) PURE;
	STDMETHOD (OnKeyUp)(THIS_ IUnknown* inSource, PlatformEvent* inEvent) PURE;
	STDMETHOD (OnMouseDown)(THIS_ IUnknown* inSource, PlatformEvent* inEvent) PURE;
	STDMETHOD (OnMouseUp)(THIS_ IUnknown* inSource, PlatformEvent* inEvent) PURE;
};


#ifdef BDIAMOND
#include "BDInterfaces.h"
#endif

#endif

